// Author: Alessandro A. Nacci
// alessandro.nacci@polimi.it
// IEIM 2014


#include <stdio.h>

#define R 3
#define C 3

void foo1(int*);
void foo2(int mat[R][C]);
void foo3(int mat[][C]);

int main()
{

	int mat[R][C];

	// Abbiamo definitio una matrice mat di rimensione RxC.
	// Ricordate che 'mat' e' una variabile puntatore che punta al primo
	// elemnto della matrice. Detto in un altro modo, il contenuto della
	// variabile mat e' l'indirizzo di memoria del primo elemento della matrice. 

	// Con questo programma mostriamo che ci sono varie modalità di passaggio 
	// di una matrice ad una funzione.
	//
	// In tutti i casi il passasggio avviene per INDIRIZZO e NON per copia.

	foo1(mat);
	// Se volete, implementatate la stampa di questa matrice a schermo 
	// per vedere il risultato dopo foo1

	foo2(mat);
	// Se volete, implementatate la stampa di questa matrice a schermo 
	// per vedere il risultato dopo foo2


	foo3(mat);
	// Se volete, implementatate la stampa di questa matrice a schermo 
	// per vedere il risultato dopo foo3


}


void foo1(int *mat)
{

	/* 
		La funzione foo1 richiede come parametro in ingresso un puntatore ad intero.
		Il programmatore sa implicitamente (sottoindende) che quel puntatore punta al 
		primo elemento di una matrice di dimensione RxC.
		Dato che non viene esplicitato con il parametro che si tratta di una matrice
		si dovrà accedere agli elementi della matrice stessa calcolando a mano
		lo spiazzamento (l'indirizzo della cella (i,j)) con la formula: 

			mat+i*C+j		==> state calcolando un indirizzo
			*(mat+i*C+j)	==> accedete al valore all'indirizzo tra parentesi

		Ricordate che 'mat' e' l'indirizzo del valore della prima cella della matrice
		di nome 'mat'.

		Il passaggio dei valori avviene ovviamente per indirizzo, quindi le modifiche
		apportate a mat risultano visibili anche dal chiamante (in questo caso il main).

		Effettaundo il passaggio della matrice con questa modalità (tenendo implicita
		per la funziona che il parametro e' in realtà una matrice), il compilatore in
		genere restituirà dei "WARNING" in quanto :
			- da un lato è molto facile sbagliare ed accedere ad aree di memoria
			che non sono parte della vostra matrice
			- dall'altro il compilatore, dato che all'interno della funzione, non 
			ha alcuna informazione relativamente alla matrice, non può effettuare 
			nessun controllo, e quindi vi dice di stare attenti. 

	*/

	int i, j;
	for(i=0;i<R;i++)
		for(j=0;j<C;j++)
			*(mat+i*C+j) = (i+j) * 2;
}




void foo2(int mat[R][C])
{

	/*

		La funzione foo2 esplicita tra i parametri che vuole una matrice di dimensione RxC.
		Avendo esplicitato cosa la funzione riceve in ingresso non è necessario calcolare 
		a mano gli spiazzamenti e quindi e' possibile accedere tranquillamente ai dati 
		della matrice con la solita sintassi 

			mat[i][j]

		NOTA BENE: anche in questo caso il passaggio NON avviene per copia ma per indirizzo!
				Questo vuol dire che quando modificate un valore della matrice 'mat' in questa
				funzione, le modifiche saranno visibili anche dal chiamante (in questo caso il main).

		Esplicitare che state passando una matrice aiuta il compilatore a trovare alcuni possibili
		errori, come per esempio, accessi ad aree di memoria fuori della matrice (indici di acesso
		alla matrice piu' grandi della dimensione della matrice)
	
	*/

	int i, j;
	for(i=0;i<R;i++)
		for(j=0;j<C;j++)
			mat[i][j] = (i+j) * 3;
}

void foo3(int mat[][C])
{

	/*

		La funzione foo3 esplicita tra i parametri che vuole una matrice con un numero di righe 
		a piacere ed un numerod di colonne pari a C.

		Dato che avete esplicitato che si tratta di una matrice, potete accederci con la comoda
		sintassi

			mat[i][j]

		NOTA BENE: anche in questo caso il passaggio NON avviene per copia ma per indirizzo!
				Questo vuol dire che quando modificate un valore della matrice 'mat' in questa
				funzione, le modifiche saranno visibili anche dal chiamante (in questo caso il main).

		Non e' strettamente necessario esplicitare il numero di righe in quanto questo dato 
		non è utile al fine del calcolo dello spiazzamento (che in questo caso il complilatore 
		svolge per voi). Se ci pensate infatti, per "linearizzare" una matrice, vale a dire, 
		per passare dalla rappresentazione matriciale alla rappresentazione vettoriale necessaria
		per salvare la matrice in memoria, vi serve solo sapere quando la matrice e' larga. Per
		capire bene questo aspetto, fate riferimento alle slides

			9. 10 Aprile 2014 - Puntatori

		sul sito del Prof. Santamborogio. 

			http://home.deib.polimi.it/santambr/dida/ieim/2014/docs.htm

	
	*/


	int i, j;
	for(i=0;i<R;i++)
		for(j=0;j<C;j++)
			mat[i][j] = (i+j) * 4;
}